%% 
%   you do not need to run the whole script
%   remember you can use ctrl+Enter to execute one block at a time
%%


%##########################################################################
%              PREPARATIONS
%##########################################################################
%% Parameters

K = 50
%%

%load mesh
M1 = meshload('shapes/lion-03_v1002.off');
M2 = meshload('shapes/lion-09_v1002.off');

% compute geodesic distance matrix (requires presence of file
% 'distmatrix.mexa64')
%D1 = distmatrix([M1.X M1.Y M1.Z]',M1.tri');
%D2 = distmatrix([M2.X M2.Y M2.Z]',M2.tri');
load('shapes/lions_distances.mat')

numberOfVertices1 = size(M1.X,1);
numberOfVertices2 = size(M2.X,1);
%%
%%compute FPS on the 
%%visualize FPS using metrics D1,D2

figure
subplot(1,2,1);
v1 = randi(numberOfVertices1);
S1 = metricfps(K, v1, D1);
f1 = metricvoronoi(S1,D1);
meshplot(M1,f1);
%cloudplot([M1.X M1.Y M1.Z],f1);
hold on
scatter3(M1.X(S1),M1.Y(S1),M1.Z(S1),25,[1 1 1],'o','filled');
hold off

subplot(1,2,2);
v2 = randi(numberOfVertices2);
S2 = metricfps(K, v2, D2);
f2 = metricvoronoi(S2,D2);
meshplot(M2,f2);
%cloudplot([M2.X M2.Y M2.Z],f2);
hold on
scatter3(M2.X(S2),M2.Y(S2),M2.Z(S2),25,[1 1 1],'o','filled');
hold off

%%
%##########################################################################
%              Matching
%##########################################################################

%metrics restricted to the FPS
D1S = D1(S1,S1);
D2S = D2(S2,S2);

% compute cost matrix C
C = distortioncostmatrix(D1S,D2S,1);
x0 = ones(size(C,1),1)./size(C,1);
xSpectral = spectralrelax(C, x0, 100, 1e-15);
xGame = gametheoretic(C,x0, 10000, 1e-6);

%% 
%##########################################################################
%              Visualization (spectral relaxation)
%##########################################################################

%pick a vertex for the spectral relaxation
v0 = 1;

figure
subplot(1,2,1);
meshplot(M1,double(f1==v0));
%cloudplot([M1.X M1.Y M1.Z],f1);
hold on
scatter3(M1.X(S1),M1.Y(S1),M1.Z(S1),25,[1 1 1],'o','filled');
hold off

m = xSpectral(v0,:);
subplot(1,2,2);
meshplot(M2,m(f2));
%cloudplot([M2.X M2.Y M2.Z],f2);
hold on
scatter3(M2.X(S2),M2.Y(S2),M2.Z(S2),25,[1 1 1],'o','filled');
hold off
%% 
%##########################################################################
%              Visualization (game-theoretic relaxation)
%##########################################################################
epsilon =  1e-2;

M1_scale = 1/max(max(abs([M1.X M1.Y, M1.Z])));
M1scaled.X = M1.X.*M1_scale;
M1scaled.Y = M1.Y.*M1_scale;
M1scaled.Z = M1.Z.*M1_scale;
M1scaled.tri = M1.tri;

M2_translate = [1 0 0];
M2_scale = 1/max(max(abs([M2.X M2.Y, M2.Z])));
M2scaled.X = (M2.X.*M2_scale)+M2_translate(1);
M2scaled.Y = (M2.Y.*M2_scale)+M2_translate(2);
M2scaled.Z = (M2.Z.*M2_scale)+M2_translate(3);
M2scaled.tri = M2.tri;

linesX = [];
linesY = [];
linesZ = [];

for i=1:K
    for j=1:K
        if (xGame(i,j)>epsilon)
           linesX = [linesX  M1scaled.X(S1(i))];
           linesY = [linesY  M1scaled.Y(S1(i))];
           linesZ = [linesZ  M1scaled.Z(S1(i))];
           linesX = [linesX  M2scaled.X(S2(j))];
           linesY = [linesY  M2scaled.Y(S2(j))];
           linesZ = [linesZ  M2scaled.Z(S2(j))];
        end
    end
end

figure
meshplot(M1scaled,M1.X);
shading interp
%cloudplot([M1.X M1.Y M1.Z],f1);
hold on
meshplot(M2scaled,M2.X);
shading interp
line(linesX',linesY',linesZ');
hold off

